class Graph:
    ''' a graph represented using an adjacency list ''' 
    
    def __init__(self, num_vertices, is_directed = False):
        ''' 
        Create a new graph with num_vertices vertices. 
        If is_directed = True is specified, then the graph is directed.
        ''' 
        self.num_vertices = num_vertices        
        self.vertices = [[] for i in range(num_vertices)]
        self.label_matrix = [['']*num_vertices for i in range(num_vertices)]
        self.is_directed = is_directed
    
    def add_edge(self, v1, v2, label):
        ''' 
        Add an edge from v1 to v2, labelling it with the given label. 
        Vertices are integers in the range 0 to num_vertices-1, 
        and label is a single character.''' 
        self.vertices[v1].append((v2, label))
        self.label_matrix[v1][v2] = label
        if not self.is_directed:
            self.vertices[v2].append((v1, label))
            self.label_matrix[v2][v1] = label
        
    def get_neighbours(self, v):
        ''' 
        Return a copy of the list of vertices that are adjacent to vertex v. 
        More precisely, the list returned is a list of 2-element 
        tuples (v2, label) where v2 is the vertex adjacent to v, 
        and label is the character labelling the edge from v to v2. 
        Vertices are integers in the range 0 to num_vertices-1.
        ''' 
        return self.vertices[v][:]
    
    def get_edge_label(self, v1, v2):
        ''' return the label (character) on the edge from v1 to v2 '''
        return self.label_matrix[v1][v2]
    

